/* Includes */
#include <avr/io.h>
#include <avr/signal.h>
#include <uart.h>
#include <pga2310.h>
#include <stdlib.h>


/* UART Buffer Defines */
#define UART_RX_BUFFER_SIZE 2     /* 2,4,8,16,32,64,128 or 256 bytes */

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
	#error RX buffer size is not a power of 2
#endif


#define BAUD        38400UL
#define UBRR_BAUD   ((F_CPU/(16UL*BAUD))-1)

//------------------------------------------------------------------------------
// LOCAL VARIABLES
//------------------------------------------------------------------------------
static unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static volatile unsigned char UART_RxHead;
static volatile unsigned char UART_RxTail;


//------------------------------------------------------------------------------
void Bios_InitUART(void)			                      // USART initialisieren
{
	  //sei();	                                        // Interruptbehandlung aktivieren
    UBRRH = (uint8_t) (UBRR_BAUD>>8);				      // Baudrate einstellen (Normaler Modus)
    UBRRL = (uint8_t) (UBRR_BAUD & 0x0ff);
    UCSRB = (1<<RXEN)|(1<<TXEN)|(1 << RXCIE);     // Aktivieren von receiver und transmitter
    UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);		  // Einstellen des Datenformats: 8 Datenbits, 1 Stoppbit
}
//------------------------------------------------------------------------------

void Bios_PutChar_P(uint8_t PutChar_P_temp)			                      // Einen Wert Senden  
{
  char Bios_PutChar_P_c[4];
  itoa( PutChar_P_temp, Bios_PutChar_P_c, 10 ); // 10 fuer radix -> Dezimalsystem
  Bios_PutString( Bios_PutChar_P_c );
}



//------------------------------------------------------------------------------
void Bios_PutChar(unsigned char c)			            // Ein Zeichen senden
{
    while(!(UCSRA & (1 << UDRE)));   	            // warte, bis UDR bereit 
    UDR = c;    						                        // sende Zeichen 
}
//------------------------------------------------------------------------------
void Bios_PutString(char *s)				                // Einen String senden
{
    while (*s)
    {                                               // so lange *s != NULL 
        Bios_PutChar(*s);
        s++;
    }
}
//------------------------------------------------------------------------------
SIGNAL(SIG_USART0_RX)
{
	unsigned char data;
	unsigned char tmphead;

	data = UDR;                 // Read the received data
	// Calculate buffer index
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead;      // Store new index

	if ( tmphead == UART_RxTail )
	{
		// ERROR! Receive buffer overflow
	}
	
	UART_RxBuf[tmphead] = data; // Store received data in buffer 
}

/* Read and write functions */
unsigned char receiveByte( void )
{
	unsigned char tmptail;
	
	while ( UART_RxHead == UART_RxTail );  /* Wait for incomming data */
	
	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;/* Calculate buffer index */
	
	UART_RxTail = tmptail;                /* Store new index */
	
	return UART_RxBuf[tmptail];           /* Return data */
}

unsigned char UARTdataReceived( void )
{
	if ( UART_RxHead == UART_RxTail )
	{
		return 0;
	}else{
		return 1;
	}
}


